#!/usr/bin/env python3

import dialog
import os
import json
import datetime
import sys
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.hostname import hostname_util
from py_tools.config.config import Config
from py_tools.puppetconf import puppet_util
from py_tools.logger import logger

log = logger.Logger()

def create_dialog():
    """
    Создать объект диалога
    """
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Редактирование реестра"))
    return d

def load_json(cashreg_filename):
    """
    Загрузить json-объект из файла cashreg_filename
    """
    log.info("Загрузка параметров реестра из файла %s" % cashreg_filename)
    if os.path.exists(cashreg_filename):
        try:
            obj = json.load(open(cashreg_filename))
            log.info("Загрузка успешно завершена")
            return obj
        except (ValueError, IOError):
            log.error("Ошибка чтения данных")
            return None
    log.warn("Файл %s" % cashreg_filename + " не существует")
    return None

def create_object_from_json(cashreg_object, cashreg_json):
    """
    Создать cashreg_object из JSON-объекта
    """
    for field in cashreg_object:
        if field in cashreg_json:
            cashreg_object[field] = cashreg_json[field]
    return cashreg_object

def show_edit_form(d, cashreg_object, first_run):
    """
    Вывести форму редактирования данных реестра
    """
    text = "Заполните параметры реестра\n"
    if not first_run:
        if cashreg_object["shiftBeg"]:
            try:
                shift = datetime.datetime.strptime(cashreg_object["shiftBeg"], "%Y-%m-%dT%H:%M:%S.%f")
                text += "Начало смены: " + shift.strftime("%d.%m.%Y %H:%M")
            except ValueError:
                text += "Начало смены: " + str(cashreg_object["shiftBeg"])
        else:
            text += "Смена закрыта"

    is_valid_object = False
    code = d.OK
    tag = []

    if not first_run:
        elements = [
            ("Код магазина:", 1, 1, cashreg_object['shopCode'], 1, 15, 17, 0),
            ("Код кассы:", 2, 1, cashreg_object['cashCode'], 2, 15, 17, 0),
            ("Номер смены:", 3, 1, str(cashreg_object['shiftNumber']), 3, 15, 17, 0),
            ("Номер чека:", 4, 1, str(cashreg_object['checkNumber']), 4, 15, 17, 0)
        ]
    else:
        elements = [
            ("Код магазина:", 1, 1, cashreg_object['shopCode'], 1, 15, 17, 0),
            ("Код кассы:", 2, 1, cashreg_object['cashCode'], 2, 15, 17, 0)
        ]

    while not is_valid_object and code == d.OK:
        code, tag = d.form(
            text=text,
            height=12, width=38, form_height=5,
            elements=elements,
            ok_label = "Далее",
            cancel_label = "Закрыть"
        )
        if code == d.OK:
            if parse_object_from_tag(cashreg_object, tag) is not None:
                is_valid_object = True
            else:
                d.msgbox("Ошибка: в номере смены и номере чека должны быть только цифры", height=8)

    return code, tag

def save_json(cashreg_filename, cashreg_object):
    """
    Сохранить данные реестра cashreg_object в JSON-формате в файл cashreg_filename
    """
    log.info("Сохранение параметров реестра в файл %s" % cashreg_filename)
    json.dump(cashreg_object, open(cashreg_filename, "w"))
    log.info("Сохранение успешно завершено")

def parse_object_from_tag(cashreg_object, tag):
    """
    Заполнить поля cashreg_object из tag, который вернула форма
    """
    cashreg_object['shopCode'] = tag[0]
    cashreg_object['cashCode'] = tag[1]
    if len(tag) > 2:
        if tag[2].isdigit():
            cashreg_object['shiftNumber'] = int(tag[2])
        else:
            return None
        if tag[3].isdigit():
            cashreg_object['checkNumber'] = int(tag[3])
        else:
            return None
    return cashreg_object

def is_valid(d, cashreg_object):
    if not cashreg_object["checkNumber"].isdigit():
        d.msgbox("Ошибка: в номере чека могут быть только цифры")
        return False
    if not cashreg_object["shiftNumber"].isdigit():
        d.msgbox("Ошибка: в номере смены могут быть только цифры")
        return False
    return True

class SafeDict(dict):
    def __missing__(self, key):
        return '%(' + key + ')s'

def generate_hostname(cashreg_object):
    confName = "/linuxcash/cash/conf/hostnamegen.conf"
    config = Config([confName])

    if not config.getBool('main', 'generate', False):
        return

    macroVars = {'cashcode': cashreg_object['cashCode'],
                 'shopcode': cashreg_object['shopCode']}

    for key in macroVars:
        try:
            macroVars[key] = int(macroVars[key])
        except:
            macroVars[key] = str(macroVars[key])
            pass

    hostname = config.getString('main', 'hostname', 'cash-%(shopcode)s-%(cashcode)s.localdomain')
    hostname = hostname % SafeDict(macroVars)
    log.info("Сохранение имени хоста: %s" % hostname)
    hostname_util.save_hostname(hostname, "/etc/hostname", "/etc/hosts")
    log.info("Сохранение успешно завершено")

    certName = config.getString('main', 'certname', 'cash-%(shopcode)s-%(cashcode)s')
    certName = certName % SafeDict(macroVars)
    log.info("Установка имени сертификата")
    puppet_util.setCertName(certName)
    log.info("Имя сертификата успешно установлено")

def main():
    """
    Главная функция
    """
    log.info("Редактирование реестра")
    if len(sys.argv) >= 2:
        cashreg_filename = sys.argv[1]
    else:
        cashreg_filename = "/linuxcash/cash/data/cash.reg"
    cashreg_object = {
        'cashCode': "1",
        'checkNumber': 0,
        'shiftBeg': "",
        'shiftNumber': 1,
        'shopCode': "1"
    }
    json_object = load_json(cashreg_filename)
    if json_object is not None:
        cashreg_object = create_object_from_json(cashreg_object, json_object)
    else:
        log.error("Не удалось загрузить параметры реестра")
    d = create_dialog()

    first_run = sys.argv[2] if len(sys.argv) == 3 else False

    run = True
    while run:
        code, tag = show_edit_form(d, cashreg_object, first_run)

        if code == d.OK:

            if first_run:
                text = "Код магазина: " + cashreg_object['shopCode'] + \
                    "\nКод кассы: " + cashreg_object['cashCode']
            else:
                text = "Код магазина: " + cashreg_object['shopCode'] + \
                    "\nКод кассы: " + cashreg_object['cashCode'] + \
                    "\nНомер смены: " + str(cashreg_object['shiftNumber']) + \
                    "\nНомер чека: " + str(cashreg_object['checkNumber'])

            code = d.msgbox(
                text,
                width=38,
                height=12,
                title = "Завершение настройки",
                ok_label = "Завершить",
                help_button = "enable",
                help_label = "Назад"
            )
            if code == d.OK:
                cashreg_object = parse_object_from_tag(cashreg_object, tag)
                save_json(cashreg_filename, cashreg_object)
                generate_hostname(cashreg_object)
                log.info("Настройки реестра:\n" + text)
                log.info("Настройки успешно сохранены")
                d.msgbox("Настройки успешно сохранены", height = 6)
                run = False
        else:
            run = False
    log.info("Завершение редактирования реестра")

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    try:
        main()
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение редактирования настроек реестра")
