#!/usr/bin/env python3

import configparser
import dialog
import online_initialization_util
import json
import os
import re
import requests
import socket
import subprocess
import sys

sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger
from py_tools.netconfig import network_util
from py_tools.puppetconf import puppet_util

class BadClusterIdException(Exception):
    pass

class BadIPException(Exception):
    pass

class WrongStatusException(Exception):
    pass

log = logger.Logger()

def checkInputData(ip, port):
    """
    Проверить введенные данные по маске
    :param ip: ip адрес
    :param port: номер порта
    :return: 2 булевых значения: 1 - валидность ip адреса, 2 - валидность номера порта
    """
    ipRes = False
    portRes = False
    ipreg = '([0-9]{1,3}[\.]){3}[0-9]{1,3}'
    numberreg = '^[0-9]+$'
    if re.match(ipreg, ip) is not None:
        ipRes = True
    if re.match(numberreg, port):
        portRes = True
    return ipRes, portRes

def createDialog():
    """
    Создать объект диалога
    """
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Настройка по сети"))
    return d

def inputDataForm(d, values):
    """
    Вывести форму ввода данных
    :param d: объект диалога
    :return: два занчения: 1 - результат диалога, 2 - введенные данные в виде словаря
    """
    code = d.OK
    srvConf = values
    run = True
    while run:
        code, tag = d.form(
            text='Введите данные',
            height=12, width=40, form_height=5,
            elements=[
                ("Адрес сервера:", 1, 1, srvConf['address'], 1, 17, 17, 0),
                ("Порт:", 2, 1, srvConf['port'], 2, 17, 17, 0),
                ("Номер кластера:", 3, 1, srvConf['clusterID'], 3, 17, 17, 0)
            ],
            ok_label="Далее",
            cancel_label="Закрыть"
        )
        if code == d.OK:
            srvConf['address'] = tag[0]
            srvConf['port'] = tag[1]
            srvConf['clusterID'] = tag[2]
            ipValid, portValid = checkInputData(tag[0], tag[1])
            if not (ipValid and portValid):
                text = ''
                if not ipValid:
                    text = 'Адрес сервера задан неверно'
                if not portValid:
                    text = text + '\nПорт задан неверно'
                messageBox(d, 'Ошибка ввода', text)
                continue
        run = False
    return code, srvConf

def showQuestionForm(d, question):
    """
    Вывести форму с вопросом
    :param d: объект диалога
    :return: результат диалога
    """
    return d.yesno(question)

def messageBox(d, title, text, okLabel='OK', width=38, height=12):
    """
    Вывести форму с сообщением
    :param d: объект диалога
    :param title: заголовок
    :param text: текст сообщения
    :param okLabel: надпись для кнопки ОК
    :param width: ширина формы
    :param height: высота формы
    :return: результат диалога
    """
    return d.msgbox(
        text,
        width=width,
        height=height,
        title=title,
        ok_label=okLabel
    )

def infoBox(d, title, text, width=38, height=12):
    """
    Вывести форму с оповещением
    :param d: объект диалога
    :param title: заголовок
    :param text: текст сообщения
    :param width: ширина формы
    :param height: высота формы
    """
    d.infobox(
        text,
        width=width,
        height=height,
        title=title
    )

def getRequestData(clusterID):
    """
    Получить данные для post запроса на сервер
    :param clusterID: номер кластера
    :return: словарь с данными для отправки на сервер
    """
    shell = subprocess.Popen('/linuxcash/cash/bin/license_info -i', shell=True, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE, universal_newlines=True)
    hardware, err = shell.communicate()
    hardware = hardware.replace('\n', '')
    with open('/linuxcash/cash/data/cash.reg', 'r') as file:
        cashreg = json.load(file)
    data = {'shopCode': cashreg['shopCode'],
            'cashCode': cashreg['cashCode'],
            'clusterId': clusterID,
            'hardwareId': hardware}
    return data

address = ''
true_socket = socket.socket
def bound_socket(*a, **k):
    sock = true_socket(*a, **k)
    sock.bind((address, 0))
    return sock

def choiceInterface():
    choices = []
    interfaces = {}

    for choice in network_util.getIPs().split("\n"):
        tmp = choice.split(": ")
        if tmp[0].find("lo") == 0:
            continue

        choices.append(tuple([tmp[0], tmp[1]]))
        interfaces[tmp[0]] = tmp[1]
        
    if len(choices) <= 1:
        log.info("Выбор сетевого интерфейса не требуется")
        return 

    log.info("Выбор сетевого интерфейса для выполнения запроса")
    d = createDialog()
    code,result = d.menu(text="Выбор сетевого интерфейса", width=40, choices=choices)

    global address
    if code != d.OK:
        address = ''
        bound_socket()
        return 

    log.info("Выбран интерфейс: %s %s" % (result, interfaces[result]))
    address = interfaces[result]
    socket.socket = bound_socket

def getConf(address, port, data, dialog):
    """
    Отправить на сервер post запрос, и получить данные для настройки puppet и настройки кассы
    :param address: адрес кассового сервера
    :param port: порт
    :param data: данные для отправки на сервер
    :param dialog: объект диалога
    :return: словарь с настройками для puppet, настройками для кассы
    """
    force = False
    while True:
        try:
            jdata = json.dumps(data)
            log.info('Отправляем запрос на КС: ' + jdata)
            infoBox(dialog, 'Получение данных', 'Соединение с сервером', width=25, height=5)
            headers = {'content-type': 'application/json'}
            params = {}
            if force:
                params.update({'force': 'true'})

            choiceInterface()
                
            response = requests.post('http://' + address + ':' + port + '/CSrest/rest/v2/cashes/initialization', headers=headers,
                                     data=jdata, timeout=30, params=params)

            log.info('Получен ответ: ' + response.text)
            try:
                answer = json.loads(response.text)
            except Exception as e:
                raise Exception("Не удалось преобразовать полученные данные к формату JSON (%s)", str(e))
            if response.status_code == 200:
                # передали clusterId, но такого нет
                if answer['code'] == 1:
                    raise Exception('Невозможно найти кластер с номером %s' % data['clusterId'])
                # не нашли магазин по shopCode и clusterId
                if answer['code'] == 2:
                    raise Exception('Невозможно найти магазин по коду %s и номеру кластера %s' % (data['shopCode'],
                                    data['clusterId']))
                # нашли несколько магазинов по shopCode и clusterId
                if answer['code'] == 3:
                    raise Exception('Найдено несколько магазинов по коду %s и номеру кластера %s' % (data['shopCode'],
                                    data['clusterId']))
                # не нашли кассу
                if answer['code'] == 4:
                    raise Exception('Невозможно найти кассу %s по коду магазина %s' % (data['cashCode'],
                                    data['shopCode']))
                # у найденной кассы заполнен IP адрес и он не совпадает с IP адресом, с которого выполнен запрос
                if answer['code'] == 5:
                    errortext = 'IP адрес кассы %s не совпадает со значением на сервере' % data['cashCode']
                    log.error('Ошибка: ' + errortext)
                    code = showQuestionForm(dialog, errortext + '. Хотите отредактировать настройки сети?')
                    if code == dialog.OK:
                        os.system('/linuxcash/cash/tools/tools_avail/py_tools/netconfig/extended_network_wizard.py')
                        continue
                    else:
                        raise BadIPException(errortext)
                # переданный hardwareId не совпадает hardwareId на кассе и не передан параметр force=true
                if answer['code'] == 10:
                    errortext = 'HardwareId кассы %s не совпадает со значением на сервере' % data['cashCode']
                    log.error('Ошибка: ' + errortext)
                    code = showQuestionForm(dialog, errortext + '. Перезаписать значение?')
                    if code == dialog.OK:
                        force = True
                        continue
                    else:
                        raise BadClusterIdException(errortext)
                # получение данных прошло успешно, code == 0
                return answer['initializeInfo'], response.status_code
            else:
                raise WrongStatusException('status: ' + str(response.status_code) + '\nerror: ' + answer['error'] +
                                '\nmessage: ' + answer['message'] + '\npath: ' + answer['path'])
        except requests.exceptions.Timeout:
            raise Exception('Истекло время ожидания соединения с сервером')
        except requests.exceptions.ConnectionError:
            raise Exception('Ошибка соединения с сервером')

def writePuppetSettings(address, port, certificate):
    """
    Сохранить настройки puppet
    :param address: адрес puppet сервера
    :param port: порт puppet сервера
    :param certificate: сертификат puppet сервера
    """
    hosts = '/etc/hosts'
    puppet = '/etc/puppet/puppet.conf'

    write = True
    replace = ''
    with open(hosts, 'r') as hostsfile:
        data = hostsfile.read()
    for line in data.split('\n'):
        if len(line) > 0 and line[0] != '#':
            split = line.split(' ')
            if split[0] == address:
                write = False
                break
            if len(split) > 1 and split[1] == 'puppet':
                replace = split[0]
    if write:
        if replace:
            data = data.replace(replace, address)
        else:
            data = data + '\n#Puppet ip address\n' + address + ' puppet\n'
        with open(hosts, 'w') as hostsfile:
            hostsfile.write(data)

    config = configparser.ConfigParser()
    config.read(puppet)
    if 'agent' not in config.sections():
        config.add_section('agent')
    config.set('agent', 'certname', certificate.lower())
    config.set('agent', 'server', 'puppet')
    config.set('agent', 'masterport', port)
    with open(puppet, 'w') as configfile:
        config.write(configfile)

def main():
    """
    Главная функция
    """
    log.info("Получение настроек с КС")

    maintainer_menu = sys.argv[1] if len(sys.argv) == 2 else False

    exitcode = 0
    run = True
    srvConf = {'address': '',
               'port': '8080',
               'clusterID': ''}
    while run:
        d = createDialog()
        code = showQuestionForm(d, "Получить настройки с кассового сервера?")
        if code != d.OK:
            if not maintainer_menu:
                online_initialization_util.init_shift_data(0)
            run = False
            exitcode = 1
            continue
        code, srvConf = inputDataForm(d, srvConf)
        if code == d.OK:
            try:
                answer, status = getConf(srvConf['address'], srvConf['port'],
                                       getRequestData(srvConf['clusterID']), d)

                text = 'Адрес puppet сервера: ' + answer['puppetServerIp'] + '\n' + \
                    'Порт puppet сервера: ' + str(answer['puppetServerPort']) + '\n' + \
                    'Сертификат puppet: ' + answer['cashId']
                messageBox(d, 'Puppet сервер', text, width=42, height=10)
                infoBox(d, 'Настройка', 'Проверка связи с puppet сервером', 33, 5)
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                result = sock.connect_ex((answer['puppetServerIp'], int(answer['puppetServerPort'])))
                writePuppetSettings(answer['puppetServerIp'], str(answer['puppetServerPort']), answer['cashId'])
                if status == 200:
                    if not 'shiftNum' in answer or not 'checkNum' in answer:
                        message = 'Не удалось получить данные о текущей смене c кассового сервера. Заполните их вручную'
                        log.error('Ошибка: ' + message)
                        messageBox(d, 'Ошибка', message)
                        online_initialization_util.init_shift_data(0)
                    else:
                        online_initialization_util.init_shift_data(status, answer['shiftNum'], answer['checkNum'],
                                answer.get('shiftTimeBeg'), answer.get('shiftTimeEnd'))
                if result != 0:
                    raise Exception('Puppet сервер недоступен')
                infoBox(d, 'Настройка', 'Запуск сервиса puppet', 30, 5)
                shell = subprocess.Popen('puppet agent -t --detailed-exitcodes', shell=True, stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE, universal_newlines=True)
                output, err = shell.communicate()
                log.info('`puppet agent -t` - exit code: %d' % shell.returncode)
                log.info('`puppet agent -t` output:\n' + output)
                log.error('`puppet agent -t` errors:\n' + err)
                if shell.returncode != 0 and shell.returncode != 2:
                    raise Exception('Не удалось применить настройки puppet')
                puppet_util.startService()
                run = False

            except BadClusterIdException as e:
                run = False
                online_initialization_util.init_shift_data(400)
                exitcode = 1
                continue
            except BadIPException as e:
                run = False
                online_initialization_util.init_shift_data(400)
                exitcode = 1
                continue
            except WrongStatusException as e:
                log.error('Ошибка: ' + str(e))
                run = False
                messageBox(d, 'Ошибка', str(e))
                online_initialization_util.init_shift_data(400)
                exitcode = 1
                continue
            except Exception as e:
                log.error('Ошибка: ' + str(e))
                messageBox(d, 'Ошибка', str(e))

    log.info("Завершение получения настроек")
    exit(exitcode)

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    try:
        main()
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение получения настроек с кассового сервера")
        exit(1)
