#!/usr/bin/env python3

import dialog
import json
import os
import datetime
import sys

sys.path.append("/linuxcash/cash/tools/tools_avail")

from py_tools.logger import logger

log = logger.Logger()

def create_dialog():
    """
    Создать объект диалога
    """
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Редактирование реестра"))
    return d

def load_json(cashreg_filename):
    """
    Загрузить json-объект из файла cashreg_filename
    """
    log.info("Загрузка параметров реестра из файла %s" % cashreg_filename)
    if os.path.exists(cashreg_filename):
        try:
            obj = json.load(open(cashreg_filename))
            log.info("Загрузка успешно завершена")
            return obj
        except (ValueError, IOError):
            log.error("Ошибка чтения данных")
            return None
    log.warn("Файл %s" % cashreg_filename + " не существует")
    return None

def create_object_from_json(cashreg_object, cashreg_json):
    """
    Создать cashreg_object из JSON-объекта
    """
    for field in cashreg_object:
        if field in cashreg_json:
            cashreg_object[field] = cashreg_json[field]
    return cashreg_object

def save_json(cashreg_filename, cashreg_object):
    """
    Сохранить данные реестра cashreg_object в JSON-формате в файл cashreg_filename
    """
    log.info("Сохранение параметров реестра в файл %s" % cashreg_filename)
    json.dump(cashreg_object, open(cashreg_filename, "w"))
    log.info("Сохранение успешно завершено")

def parse_object_from_tag(cashreg_object, tag):
    """
    Заполнить поля cashreg_object из tag, который вернула форма
    """
    if tag[0].isdigit():
        cashreg_object['shiftNumber'] = int(tag[0])
    else:
        return None
    if tag[1].isdigit():
        cashreg_object['checkNumber'] = int(tag[1])
    else:
        return None
    return cashreg_object

def shift_data_edit_form(d, cashreg_object, text):
    """
    Вывести форму редактирования данных реестра
    """

    if cashreg_object["shiftBeg"]:
        try:
            shift = datetime.datetime.strptime(cashreg_object["shiftBeg"], "%Y-%m-%dT%H:%M:%S.%f")
            text += "Начало смены: " + shift.strftime("%d.%m.%Y %H:%M")
        except ValueError:
            text += "Начало смены: " + str(cashreg_object["shiftBeg"])
    else:
        text += "Смена №" + str(cashreg_object['shiftNumber'] - 1) + " закрыта"

    is_valid_object = False
    code = d.OK
    tag = []
    while not is_valid_object and code == d.OK:
        code, tag = d.form(
            text=text,
            height=12, width=38, form_height=4,
            elements=[
                ("Номер смены:", 1, 1, str(cashreg_object['shiftNumber']), 1, 14, 17, 0),
                ("Номер чека:", 2, 1, str(cashreg_object['checkNumber']), 2, 14, 17, 0)
            ],
            ok_label = "Далее",
            cancel_label = "Закрыть"
        )
        if code == d.OK:
            if parse_object_from_tag(cashreg_object, tag) is not None:
                is_valid_object = True
            else:
                d.msgbox("Ошибка: в номере смены и номере чека должны быть только цифры", height=8)

    return code, tag

def init_shift_data(status_code, shift_number=None, check_number=None, shift_beg=None, shift_end=None):
    """
    Получаем с сервера данные о смене и чеке
    """
    d = create_dialog()
    cashreg_filename = "/linuxcash/cash/data/cash.reg"
    cashreg_object = {
        'cashCode': "1",
        'checkNumber': 0,
        'shiftBeg': "",
        'shiftNumber': 1,
        'shopCode': "1"
    }
    json_object = load_json(cashreg_filename)
    if json_object is not None:
        cashreg_object = create_object_from_json(cashreg_object, json_object)
    else:
        log.error("Не удалось загрузить параметры реестра")

    if status_code == 200:
        text = "Данные о смене, полученные с кассового сервера\n"
        if shift_number is not None and shift_number >= cashreg_object['shiftNumber']:
            if shift_end is None:
                if shift_number > cashreg_object['shiftNumber']:
                    cashreg_object['shiftNumber'] = shift_number
                    cashreg_object['checkNumber'] = check_number if not check_number is None else 0
                elif check_number is not None and check_number > cashreg_object['checkNumber']:
                    cashreg_object['checkNumber'] = check_number
                else:
                    text = "В файле реестра уже содержатся актуальные данные о смене\n"
                cashreg_object['shiftBeg'] = shift_beg
            else:
                cashreg_object['shiftNumber'] = shift_number + 1
                cashreg_object['checkNumber'] = 0
                cashreg_object['shiftBeg'] = ""
        else:
            text = "В файле реестра уже содержатся актуальные данные о смене\n"
    elif status_code == 400:
        text = "Не удалось получить данные о смене с кассового сервера, заполните их вручную\n"
    else:
        text = "Заполните параметры реестра\n"

    run = True
    while run:
        code, tag = shift_data_edit_form(d, cashreg_object, text)

        if code == d.OK:
            text = "Номер смены: " + str(cashreg_object['shiftNumber']) + "\n"\
                "Номер чека: " + str(cashreg_object['checkNumber']) + "\n"

            code = d.msgbox(
                text,
                width=38,
                height=12,
                title = "Завершение настройки",
                ok_label = "Завершить",
                help_button = "enable",
                help_label = "Назад"
            )
            if code == d.OK:
                cashreg_object = parse_object_from_tag(cashreg_object, tag)
                save_json(cashreg_filename, cashreg_object)
                log.info("Настройки реестра:\n" + text)
                log.info("Настройки успешно сохранены")
                d.msgbox("Настройки успешно сохранены", height = 8)
                run = False
        else:
            run = False
    return True

