#!/usr/bin/env python3

import dialog
import os
import subprocess
import sys

sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger


log = logger.Logger()
available_processings = {
    "sberbank": {
        "package": "artix45-plugin-sb",
        "required_hardware": True,
        "name": "СберБанк"
    },
    "inpas": {
        "package": "artix45-plugin-inpas",
        "required_hardware": True,
        "name": "Инпас"
    },
    "arcus": {
        "package": "artix-arcusmultimerchant",
        "required_hardware": True,
        "name": "Аркус"
    }
}


def create_dialog():
    """
    Создать объект диалога
    """
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Установка процессингов"))
    return d


def choose_processings_dialog(d):
    """
    Показать диалог выбора процессингов для установки
    """
    (code, tags) = d.checklist(text="Выберите процессинги для установки", width=60,
                               list_height=len(available_processings.keys()),
                               choices=[(key, available_processings[key]["name"], 0)
                                        for key in available_processings.keys()])
    return tags if code == d.OK else []


def join_names(keys):
    """
    Получить список названий процессингов через запятую
        keys - список ключей процессингов
    """
    return ", ".join([available_processings[key]["name"] for key in keys])


def show_info_dialog(d, error_list, required_hardware):
    """
    Показать сообщение по итогам установки процессингов
        error_list - список ключей процессингов, при установке которых произошли ошибки
        required_hardware - список ключей процессингов, пребующих настройки оборудования
    """
    message = ("Произошла ошибка при установке следующих процессингов: %s. Попробуйте установить их вручную." %
               join_names(error_list) if error_list else "")
    message = message + ("\n\n" if error_list and required_hardware else "")
    message = message + ("Следующие процессинги требуют настройки оборудования: %s." %
                         join_names(required_hardware) if required_hardware else "")
    d.msgbox(text=message, width=60)


def install_processing(processing):
    """
    Установка и конфигурирование процессинга
        processing - ключ процессинга из справочника
    """
    log.info("Установка процессинга " + available_processings[processing]["name"])
    shell = subprocess.Popen(
        "apt-get update && apt-get --yes --force-yes install " + available_processings[processing]["package"],
        shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    res, err = shell.communicate()
    if shell.returncode == 0:
        shell = subprocess.Popen("dpkg-reconfigure " + available_processings[processing]["package"], shell=True,
                                 stdout=sys.stdout, stderr=sys.stderr)
        shell.communicate()
        log.info("Установка успешно завершена")
        return True
    else:
        log.error("В процессе установки возникла ошибка:\n" + err)
        return False


def show_install_progress(d, progress_dict):
    """
    Показать информацию о прогрессе установки процессингов
        progress_dict - список состояний установки {name: status}, где
                        name    ключ процессинга,
                        status  состояние установки (0 - в очереди, 1 - в процессе установки,
                                2 - успешно установлен, 3 - ошибка при установке)
    """
    statuses = ["Ожидание", "В процессе", "Успех", "Ошибка"]
    max_len = len(max([available_processings[key]["name"] for key in progress_dict.keys()], key=len))
    process_list = (
        available_processings[key]["name"].rjust(max_len - (max_len - len(available_processings[key]["name"])) // 2, ' ')
        + "\t\t" + statuses[progress_dict[key]] for key in progress_dict.keys())
    message = "Выполняется установка\n\n" + "\n".join(process_list)
    d.infobox(text=message, width=60)


def main():
    """
    Главная функция
    """
    log.info("Установка процессингов")
    d = create_dialog()
    install_processings = choose_processings_dialog(d)
    if not install_processings:
        exit(0)

    progress_dict = dict((key, 0) for key in install_processings)
    for processing in install_processings:
        progress_dict[processing] = 1
        show_install_progress(d, progress_dict)
        progress_dict[processing] = 2 if install_processing(processing) else 3

    end_with_error = [key for key in progress_dict.keys() if progress_dict[key] == 3]
    required_hardware = [key for key in progress_dict.keys()
                         if progress_dict[key] == 2 and available_processings[key]["required_hardware"]]
    if end_with_error or required_hardware:
        show_info_dialog(d, end_with_error, required_hardware)

    log.info("Завершение установки процессингов")


if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    try:
        main()
    except KeyboardInterrupt:
        log.warn('Неожиданное завершение установки процессингов')
        exit(1)
